#region Using declarations
using System;
using System.ComponentModel;
using System.Drawing;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	#region "chartCoord Structure"
	[Serializable()]	
	public struct chartCoord
	{
		public DateTime X;
		public double Y;
		public bool Valid;
	}
	#endregion

	#region "LineEx Class"
	[Serializable()]	
	public class LineEx
	{
		public Point P1;
		public Point P2;

		public LineEx(int x1, int y1,int x2,int y2)
		{
			P1.X = x1; P1.Y=y1; P2.X=x2; P2.Y=y2; 
		}
		public LineEx(Point p1, Point p2)
		{
			if (P2.X >= P1.X)
					{ P1=p1; P2=p2; }
			else  	{ P1=p2; P2=p1; }
		}

		public double LineSlope( )
		{ 	// m = (y2-y1)/(x2-x1) = (yn-y1)/(xn-x1)
			double m;			
						
			if (P1.X != P2.X)
				m = (double)( P2.Y - P1.Y) / (double)(P2.X - P1.X); 
			else
				m = 0;
			
			return m;
		}
		
		public int CalcY( int x )   
		{	// m = (y2-y1)/(x2-x1) = (yn-y1)/(xn-x1)
			// so yn = m(xn-x1) + y1
			double m = this.LineSlope();	
			double y = m * (x - P1.X) + P1.Y ;	
			return Convert.ToInt32 ( y );
		}	// Why doesn't this work ??
		
		public void ExtendLine( int x )
		{		
			if (x < P1.X) 
				return ;
			
			double m = this.LineSlope();
			/* if (P1.X != P2.X)
				m = (double)( P2.Y - P1.Y) / (double)(P2.X - P1.X);
			else
				m = 0; */
			
			//double b = P1.Y - ( m * P1.X ) ;
			//double y = (m * x) + b ;		
			double y = m * (x - P1.X) + P1.Y ; 
			
			P2.X = x;
			P2.Y = Convert.ToInt32 ( y );
			//P2.Y = this.CalcY( x );     // doesn't work the same, why not??
		}
		
		public LineEx CreateParalell (Point P)
		{
			// Y = mX + b
			// X = (Y-b)/m
			
			if (P1.X == 0)
				return new LineEx(P.X, P.Y, P.X, P.Y);
			
			double m = this.LineSlope();
			/*double m=0;
			if (P1.X != P2.X)
				m = (double)( P2.Y - P1.Y) / (double)(P2.X - P1.X);*/
			
			//double b = P.Y - ( m * P.X ) ;
			//double y = m * P2.X + b ;
			double y = m * (P2.X - P.X) + P.Y ; 
			
			Point PointX = new Point (P2.X, Convert.ToInt32(y));   //  the new end point of the new line
			
			return (new LineEx(P, PointX) );	
		}		
		
		public bool IsPointOutOfLine ( Point P, bool highs)
		{
			bool ret = false;
			
			double m = this.LineSlope();
			/*double m;
			if (P1.X != P2.X)
				m = (double)( P2.Y - P1.Y) / (double)(P2.X - P1.X);
			else
				m = 0; */
			
			//double b = P1.Y - ( m * P1.X ) ;
			//double y = m * P.X + b ;
			double y = m * (P.X - P1.X) + P1.Y ; 
			
			if (highs){
				if ( P.Y < y ) ret = true;}
			else{
				if ( P.Y > y ) ret = true;}
							
			return ret;
		}			
	}	
	#endregion
	
	#region "ChartHelper Class"
	public class ChartHelper
    {
        protected Rectangle	_chartBounds;
        protected double 	_chartMin;
        protected double 	_chartMax;
        protected double 	TickSize;

        public ChartControl chartControl;
		private string 		_debug;
		private Bars 		Bars;

        public ChartHelper(ChartControl chartCtrl, Bars bar, double tickSize)
        {
            chartControl = chartCtrl;
            Bars = bar;
            TickSize = tickSize;
        }

        public void Update(Rectangle bounds, double min, double max)
        {
            _chartBounds = bounds; _chartMin = min; _chartMax = max;
        }

        public string GetDebugInfo()
        {
            return _debug;
        }

	    public bool SessionActive()
        {
			if (chartControl != null
					&& Bars != null
					&& Bars.Count > 0
					&& Bars.MarketData != null
					&& Bars.MarketData.Connection.PriceStatus == Cbi.ConnectionStatus.Connected
					&& (Bars.MarketData.Connection.Options.Provider != Cbi.Provider.OpenTick || !(Bars.MarketData.Connection.Options as Cbi.OpenTickOptions).UseDelayedData)
					&& Bars.Session.InSession(Now, Bars.Period.Id, true))
				return true;
            return false;
        }
		
   		public DateTime Now
		{
			get { return (Bars.MarketData.Connection.Options.Provider == Cbi.Provider.Replay ? Bars.MarketData.Connection.Now : DateTime.Now); }
		}		
		
		public int CalcTickHeight ()
		{
			int tickHeight = (int)Math.Round ( Math.Round (((_chartBounds.Bottom - _chartBounds.Y) / (_chartMax - _chartMin) ) ,0) * TickSize ,0);
			return tickHeight ; 
		}
		
		public double CalcProRataVolume (double currentVolume, int period, int waitTime)
		{
			double _prv = currentVolume;
			
			if (!SessionActive())
				return _prv;

			double _wait = (double) waitTime /( period * 60);
			
			if (Bars.PercentComplete > _wait ) 
				_prv = (currentVolume / Bars.PercentComplete) - (currentVolume / Bars.PercentComplete % 1);
			//		_prv = Volume[0]/Bars.PercentComplete - (Volume[0]/Bars.PercentComplete % 1);

			return _prv ;
		}
		
        public chartCoord CalcChartCoordFromCanvasCoord(int x, int y)
        {
            _debug = "";
            chartCoord coord;

            coord.Y = ConvertYtoPrice(y);
            coord.X = ConvertXToTime(x);
            coord.Valid = true;

            if ((coord.X == DateTime.MinValue) || (coord.Y <= 0))
                coord.Valid = false;

            _debug = "X= " + x + " Y= " +y + " Coord.X=" + coord.X + " Coord.Y=" + coord.Y + "Coord.Valid=" + coord.Valid;
            return coord;
        }

        public Point CalcCanvasCoordFromChartCoords(chartCoord coord)
        {
            _debug = "";
            Point canvasCoord = new Point(0, 0);
            if (chartControl == null)
                return canvasCoord;

            int barIdx;
            DateTime lastTime = Bars.Get(Bars.Count - 1).Time;

            if (coord.X <= lastTime)
            {
                barIdx = Bars.GetBar(coord.X);
                if (barIdx <= 0) return canvasCoord;

                canvasCoord.X = chartControl.GetXByBarIdx(barIdx);
                canvasCoord.Y = ConvertPriceToY(coord.Y);
            }
            else
            {
                TimeSpan rest = coord.X.Subtract(lastTime);
                int numBars = (int)Math.Round((rest.TotalMinutes / (double)(Bars.Period.Value)), 0);
                barIdx = Bars.GetBar(lastTime);
                if (barIdx <= 0) return canvasCoord;

                canvasCoord.X = chartControl.GetXByBarIdx(barIdx) + (numBars * chartControl.BarSpace);
                canvasCoord.Y = ConvertPriceToY(coord.Y);
            }

            return canvasCoord;
        }

        public chartCoord GetChartCoordBarIndex(int barIndex, bool high)
        {
            _debug = "";
            chartCoord coord;
            coord.Valid = false;
            coord.X = DateTime.MinValue;
            coord.Y = 0;

            if (barIndex < 0 || barIndex > Bars.Count)
                return coord;

            coord.X = ConvertIdxToTime(barIndex);
            if (high)
                coord.Y = Bars.Get(barIndex).High;
            else
                coord.Y = Bars.Get(barIndex).Low;

            coord.Valid = true;
            return coord;
        }

        public Point GetCanvasCoordBarIndex(int barIndex, bool high)
        {
            _debug = "";
            Point coord = new Point(0, 0);
            if (chartControl == null)
                return coord;

            double price;

            if (barIndex < 0 || barIndex > Bars.Count)
                return coord;

            if (high)
                price = Bars.Get(barIndex).High;
            else
                price = Bars.Get(barIndex).Low;

            coord.X = ConvertBarIdxToX (barIndex);
            coord.Y = ConvertPriceToY(price);

            return coord;
        }

        public int GetBarIdxFromChartCoord(chartCoord coord)
        {	// only works if coord < CurrentBar (not out in future white space)
            return Bars.GetBar(coord.X);
        }

        public int GetBarIdxFromCanvasCoord(int x)
        {
            return ConvertXtoBarIdx(x);
        }

		public int ConvertBarIdxToX ( int barIndex)
		{
			return chartControl.GetXByBarIdx(barIndex);
		}
		
        public int ConvertPriceToY(double price)
        {
            _debug = "";
            double ratio = (_chartMax - _chartMin) / (_chartBounds.Bottom - _chartBounds.Y);
            double chartY = _chartBounds.Bottom - ((price - _chartMin) / ratio);

            if (double.IsInfinity(chartY) || double.IsNaN(chartY))
                return 0;

            return Convert.ToInt32(chartY);
        }

        public double ConvertYtoPrice(int y)
        {
            _debug = "";
            int _tickLength = 0;

            if (TickSize < 1)
                _tickLength = TickSize.ToString().Length - 2;

            double chartscale = Math.Abs(_chartMax - _chartMin);
            double boundAreaScale = _chartBounds.Bottom - _chartBounds.Y;

            double ratio = (double)(chartscale) / boundAreaScale;
            double chartPrice = Math.Round(_chartMin + ((_chartBounds.Bottom - y) * ratio), _tickLength);

            return RoundPriceToTick(chartPrice);
        }
		
		public int ConvertTimeToIdx(DateTime time)   // in process
        {   //  like Pepe's other netohds, this only works on MINUTE charts
            _debug = "";
			int barIdx = 0;
            DateTime last = Bars.Get(Bars.Count - 1).Time;  // time of last bar on chart

            if (time <= last)
                return Bars.GetBar(time);
			
			TimeSpan rest = time.Subtract(last);   // diffference from now to future time
			int numBars = (int)Math.Round((rest.TotalMinutes / Bars.Period.Value), 0);   
			barIdx = (Bars.Count - 1) + numBars;
			/*
			Print ("Converting: FutureBarIdx =" + (Bars.Count -1).ToString() + " + " + numBars.ToString() + " = "
					+ barIdx.ToString() );
			Print(" & CurrentBar=" + CurrentBar.ToString());
			
			TimeSpan sessionEndTime = Bars.SessionEnd.TimeOfDay;
			TimeSpan lastTime = Bars.Get(lastBarIndex).Time.TimeOfDay;
			TimeSpan rest = sessionEndTime.Subtract(lastTime);
			int numBarsToCloseSession = (int)Math.Round((rest.TotalMinutes / Bars.Period.Value), 0);
			DateTime retTime = Bars.Get(Bars.Count - 1).Time.Add(new TimeSpan(0, numBars * Bars.Period.Value, 0));
			*/
			
            return barIdx;
        }

        public DateTime ConvertIdxToTime(int barIdx)
        {
            _debug = "";
            DateTime time = DateTime.MinValue;

            if (barIdx <= 0)
                return time;

            if (barIdx >= Bars.Count)
                time = CalcTimeOutofBounds(barIdx);
            else
                time = Bars.Get(barIdx).Time;

            return time;
        }

        private DateTime CalcTimeOutofBounds(int barIdx)
        {	//  this only works on MINUTE charts ??
            _debug = "";
            
			DateTime retTime = DateTime.MinValue;
            int lastBarIndex = Bars.Count - 1;
			TimeSpan sessionEndTime = Bars.SessionEnd.TimeOfDay;
            TimeSpan lastTime = Bars.Get(lastBarIndex).Time.TimeOfDay;

			TimeSpan rest = sessionEndTime.Subtract(lastTime);
            int numBarsToCloseSession = (int)Math.Round((rest.TotalMinutes / Bars.Period.Value), 0);

            int numBars = barIdx - lastBarIndex;
            retTime = Bars.Get(lastBarIndex).Time.Add(new TimeSpan(0, numBars * Bars.Period.Value, 0));

			#region "prev Version"
//            _debug = "";
//            DateTime retTime = DateTime.MinValue;
//            int lastBarIndex = Bars.Count - 1;
//
//
//            TimeSpan lastTime = Bars.Get(lastBarIndex).Time.TimeOfDay;
//
//            TimeSpan sessionStartTime = Bars.SessionBegin.TimeOfDay;
//            TimeSpan sessionEndTime = Bars.SessionEnd.TimeOfDay;
//            int numBarsSession = (int)Math.Round(sessionEndTime.Subtract(sessionStartTime).TotalMinutes / Bars.Period.Value, 0);
//
//            TimeSpan rest = sessionEndTime.Subtract(lastTime);
//
//            int numBarsToCloseSession = (int)Math.Round((rest.TotalMinutes / Bars.Period.Value), 0);
//
//            if (barIdx <= (lastBarIndex + numBarsToCloseSession))
//            {
//                int numBars = barIdx - lastBarIndex;
//                retTime = Bars.Get(lastBarIndex).Time.Add(new TimeSpan(0, numBars * Bars.Period.Value, 0));
//            }
//            else
//            {
//                int numBars = barIdx - lastBarIndex;
//                retTime = Bars.Get(lastBarIndex).Time.Add(new TimeSpan(0, numBars * Bars.Period.Value, 0));
//
//                //					int sessions =1+ (int) Math.Truncate ( (numBars / (double)numBarsSession)) ;
//                //					int numBarsRem = (int) ((numBars % (double)(numBarsSession))) ;
//                //					
//                //					retTime = Bars.SessionBegin.AddDays(sessions) ;
//                //					if (retTime.DayOfWeek == DayOfWeek.Saturday) retTime = retTime.AddDays (2);
//                //					if (retTime.DayOfWeek == DayOfWeek.Sunday ) retTime = retTime.AddDays (1);
//                //					
//                //					retTime = retTime.Add ( new TimeSpan (0, numBarsRem * Bars.Period.Value, 0) ) ; 
//            }
			#endregion
			
            _debug = "RetTime= " + retTime.ToString() + " lastBarIndex=" + lastBarIndex + " numBarsToCloseSession=" + numBarsToCloseSession + " barIdx=" + barIdx;
            return retTime;
        }

        public DateTime ConvertXToTime(int x)
        {
            _debug = "";
            int barIdx = ConvertXtoBarIdx(x);
            return ConvertIdxToTime(barIdx);
        }

        public int ConvertXtoBarIdx(int x)
        {
            _debug = "";
            if (chartControl == null)
                return 0;

            int numBarsOnCanvas = 0;
            int idxFirstBar = 0;
            int idxLastBar = 0;

            if (chartControl.LastBarPainted < Bars.Count)
            {
                numBarsOnCanvas = chartControl.LastBarPainted - chartControl.FirstBarPainted;
                idxFirstBar = chartControl.FirstBarPainted;
                idxLastBar = chartControl.LastBarPainted;
            }
            else
            {
                numBarsOnCanvas = Bars.Count - chartControl.FirstBarPainted;
                idxFirstBar = chartControl.FirstBarPainted;
                idxLastBar = Bars.Count - 1;
            }

            int firstBarX = chartControl.GetXByBarIdx(idxFirstBar);
            int halfBarWidth = (int)Math.Round(((double)(chartControl.BarSpace / (double)2)), 0, MidpointRounding.AwayFromZero);
            int margin = firstBarX + halfBarWidth;
            double ratio = 1 + ((x - margin) / (double)(chartControl.BarSpace));
            int numberPeriods = (int)Math.Truncate(ratio);

            int barIndex = idxFirstBar + numberPeriods;

            if (barIndex < 0)
                return 0;

            _debug = "BarIndex = " + barIndex + " NumBarsCanvas= " + numBarsOnCanvas + " FirstBar=" + idxFirstBar + " LastBar=" + idxLastBar + " Ratio = " + ratio + " NumPeriods =" + numberPeriods;
            return barIndex;
        }

        public double RoundPriceToTick(double price)
        {
            _debug = "";
            int numTotalTicks = (int)Math.Round(price / (double)TickSize, 0);
            return (numTotalTicks * TickSize);
        }

    }
	#endregion
}
