//STB_TickForce 0.0.1 2009:09:03 20:01:02$
//Copyright (c) 2009 solgracia
//Released under the GNU General Public License

// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Enter the description of your new custom indicator here
    /// </summary>
    public class STB_TickForce : Indicator
    {
        #region Variables
        // Wizard generated variables
        // User defined variables (add any user defined variables below)
        private DataSeries upTicksSeries;
        private DataSeries downTicksSeries;
        private DataSeries upIntegTicksSeries;
        private DataSeries downIntegTicksSeries;
        private DataSeries upResTicksSeries;
        private DataSeries downResTicksSeries;
        private DataSeries kForceSeries;

        private double pPrice = 0;
        private double p = 0;
        private double n = 0;
        private double rel;

        private double upIntegTicksValue = 0;
        private double downIntegTicksValue = 0;

        double _kforce = 0.2;
        private double _upTicksInBar = 0;
        private double _downTicksInBar = 0;
        private double _totTicksInBar = 0;
        private double _upIntegTicksInBar = 0;
        private double _downIntegTicksInBar = 0;

        private double _resTicksInBar = 0;
        private double _upResTicksInBar = 0;
        private double _downResTicksInBar = 0;

        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.SpringGreen, 5), PlotStyle.Bar, "UpBarsLine"));
            Add(new Plot(new Pen(Color.OrangeRed, 5), PlotStyle.Bar, "DownBarsLine"));
            Add(new Plot(new Pen(Color.DarkGreen, 2), PlotStyle.Bar, "UpDarkBarsLine"));
            Add(new Plot(new Pen(Color.DarkRed, 2), PlotStyle.Bar, "DownDarkBarsLine"));
            Add(new Line(Color.Blue, 0.0, "Neutral"));

            CalculateOnBarClose	= false;
            Overlay				= false;
            PriceTypeSupported	= false;

            upTicksSeries = new DataSeries(this);
            downTicksSeries = new DataSeries(this);

            upIntegTicksSeries = new DataSeries(this);
            downIntegTicksSeries = new DataSeries(this);

            upResTicksSeries = new DataSeries(this);
            downResTicksSeries = new DataSeries(this);

        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {

            if (CurrentBar < 1) return;

            if (CurrentBar == 0)
            {
                upTicksSeries.Set(0);
                downTicksSeries.Set(0);
                upIntegTicksSeries.Set(0);
                downIntegTicksSeries.Set(0);
                upResTicksSeries.Set(0);
                downResTicksSeries.Set(0);
                kForceSeries.Set(0);
                return;
            }

            if (FirstTickOfBar)
            {

                UpBarsLine.Set(1, _upResTicksInBar);
                DownBarsLine.Set(1, _downResTicksInBar);
                UpDarkBarsLine.Set(1, _upIntegTicksInBar);
                DownDarkBarsLine.Set(1, _downIntegTicksInBar);

                upTicksSeries.Set(0);
                downTicksSeries.Set(0);
                upIntegTicksSeries.Set(0);
                downResTicksSeries.Set(0);

                _upTicksInBar = 0;
                _downTicksInBar = 0;

                pPrice = Close[1];
                p = 0;
                n = 0;

                double _pkforce;
                if (upTicksSeries[1] == downTicksSeries[1]) _pkforce = 0;
                else _pkforce = (upTicksSeries[1] - downTicksSeries[1]) / (upTicksSeries[1] + downTicksSeries[1]);

                if (_pkforce > _kforce) upIntegTicksValue = upIntegTicksSeries[1]; // force up
                else upIntegTicksValue = 0;

                if (_pkforce < - _kforce) downIntegTicksValue = downIntegTicksSeries[1]; // force down
                else downIntegTicksValue = 0;
            }

            double _forceInBar;

            rel = Input[0] - pPrice;
            pPrice = Input[0];

            if (rel > 0) _upTicksInBar += rel;          // pos value
            if (rel < 0) _downTicksInBar -= rel;        // pos value
            _resTicksInBar = _upTicksInBar - _downTicksInBar;   // int value
            _totTicksInBar = _upTicksInBar + _downTicksInBar;   // pos value

            if (_resTicksInBar == 0) _forceInBar = 0;
            else _forceInBar = _resTicksInBar / _totTicksInBar; // int value

            if (_forceInBar > _kforce)
            {
                _upIntegTicksInBar = upIntegTicksValue + _resTicksInBar;
                _downIntegTicksInBar = 0;
            }
            else if (_forceInBar < -_kforce)
            {
                _upIntegTicksInBar = 0;
                _downIntegTicksInBar = downIntegTicksValue + _resTicksInBar;
            }
            else
            {
                _upIntegTicksInBar = 0;
                _downIntegTicksInBar = 0;
            }

            _upResTicksInBar = Math.Max(0, _resTicksInBar);
            _downResTicksInBar = Math.Min(0, _resTicksInBar);

            upTicksSeries.Set(_upTicksInBar);
            downTicksSeries.Set(_downTicksInBar);
            upIntegTicksSeries.Set(_upIntegTicksInBar);
            downIntegTicksSeries.Set(_downIntegTicksInBar);

            UpBarsLine.Set(_upResTicksInBar);
            DownBarsLine.Set(_downResTicksInBar);
            UpDarkBarsLine.Set(_upIntegTicksInBar);
            DownDarkBarsLine.Set(_downIntegTicksInBar);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries UpBarsLine
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries DownBarsLine
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries UpDarkBarsLine
        {
            get { return Values[2]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries DownDarkBarsLine
        {
            get { return Values[3]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries UpIntegTicksSeries
        {
            get { return upIntegTicksSeries; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries DownIntegTicksSeries
        {
            get { return downIntegTicksSeries; }
        }
        [Category("Parameters")]
        public double Kforce
        {
            get { return _kforce; }
            set { _kforce = value; }
        }

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private STB_TickForce[] cacheSTB_TickForce = null;

        private static STB_TickForce checkSTB_TickForce = new STB_TickForce();

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public STB_TickForce STB_TickForce()
        {
            return STB_TickForce(Input);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public STB_TickForce STB_TickForce(Data.IDataSeries input)
        {

            if (cacheSTB_TickForce != null)
                for (int idx = 0; idx < cacheSTB_TickForce.Length; idx++)
                    if (cacheSTB_TickForce[idx].EqualsInput(input))
                        return cacheSTB_TickForce[idx];

            STB_TickForce indicator = new STB_TickForce();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.SetUp();

            STB_TickForce[] tmp = new STB_TickForce[cacheSTB_TickForce == null ? 1 : cacheSTB_TickForce.Length + 1];
            if (cacheSTB_TickForce != null)
                cacheSTB_TickForce.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheSTB_TickForce = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.STB_TickForce STB_TickForce()
        {
            return _indicator.STB_TickForce(Input);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.STB_TickForce STB_TickForce(Data.IDataSeries input)
        {
            return _indicator.STB_TickForce(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.STB_TickForce STB_TickForce()
        {
            return _indicator.STB_TickForce(Input);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.STB_TickForce STB_TickForce(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.STB_TickForce(input);
        }

    }
}
#endregion
